==> core-0.3.7.tgz: package/lib/net/Response.js <==
var Message = require('./Message.js').Message;
var Class = require('../class/Class.js').Class;

var Assertions = require('../common/Assertions.js');
var Types = require('../common/Types.js');
var BinaryData = require('../data/BinaryData.js').BinaryData;

var http = require('http');

var Response = function(){
	this.Extends = Message;

	this._statusCode = null;

	this._cookieKey = 'set-cookie';

	this._httpServerResponse = null;

	this._reCharset = /charset=([A-Za-z\-0-9]+)/i;

	this._reStatusLine = /([0-9]{1,}\.[0-9]{1,}).*([0-9]{3,})/;

	this.initialize = function(params){
		if(Types.isInstanceOf(params, http.ServerResponse)){
			this._httpServerResponse = params;
		}else{
			this.parent(params);
		}
	};

	this.setStatusCode = function(code){
		Assertions.isNumber(code, Response.Exception.WRONG_STATUS_CODE);

		this._statusCode = code;

		return this;
	};

	this.getStatusCode = function(){
		return this._statusCode;
	};

	this._setBody = function(body){
        
		if(Types.isInstanceOf(body, BinaryData)){
			this._body = body;
		}else{
			var encoding, characterEncoding = null;
			if(Types.isInstanceOf(body, Buffer)){
				if(!this._isBinary()){
					encoding = BinaryData.Encoding.TEXT;

					switch(this.getCharset()){
						case 'iso-8859-2':
							characterEncoding = BinaryData.CharacterEncoding.ISO_8859_2;
                            encoding = BinaryData.Encoding.BINARY;
						break;
						case 'utf-8':
						case 'utf8':
							characterEncoding = BinaryData.CharacterEncoding.UTF8
						break;
                        default:
                            // TODO poprawic to bo to nie jest binarka
                            encoding = BinaryData.Encoding.BINARY;
                        break;   
					}
				}else{
					encoding = BinaryData.Encoding.BINARY;
				}
			}else{
				encoding = BinaryData.Encoding.TEXT;
				characterEncoding = BinaryData.CharacterEncoding.UTF8;
			}
			this._body = new BinaryData(body, encoding, characterEncoding);
		}
	};
    
	this.setBody = function(body){
        Assertions.isInstanceOf(body, BinaryData);
		this._body = body;
        return this;            
    };

    /**
     * Indicates if response content type matches given contentType parameter
     * @public
     * @param {String} contentType mime type to match (e.g. text/html, application/*)
     * @returns {Boolean} 
     */
    this.isContentType = function(contentType){
		Assertions.isString(contentType, Response.Exception.WRONG_CONTENT_TYPE);

		var contentHeader = this.getHeader('content-type');
		
		if(contentHeader){
	       var currentContentTypeParts = this.getHeader('content-type').split(";")[0].split("/");
    	   var checkedContentTypeParts = contentType.split(";")[0].split("/");
	       if( 
    	       (checkedContentTypeParts[0] == "*" || checkedContentTypeParts[0] == currentContentTypeParts[0])
        	   &&
	           (checkedContentTypeParts[1] == "*" || checkedContentTypeParts[1] == currentContentTypeParts[1])
    	   ){
        	   return true;            
	       }
		}

		return false;
    };

	this.isCharset = function(charset){
		Assertions.isString(charset, Response.Exception.WRONG_CHARSET_TYPE);

		var currentCharset = this.getCharset();

		if(!Types.isNull(currentCharset) && currentCharset.toLowerCase() == charset.toLowerCase()){
			return true;
		}

		return false;
	};

	this.getCharset = function(){
		var matches = this._reCharset.exec(this.getHeader('content-type'));

		if(matches && !Types.isUndefined(matches[1])){
			return matches[1].toLowerCase();
		}

		return null;
	};

	this.getEncoding = function(){
		if(this._isBinary()){
			return 'base64';
		}

		return this.getCharset();
	};

	this.toJson = function(){
        this.setHeader('content-length', this.getBody().length(), true);
        
		return {
			"statusCode": this._statusCode,
			"httpVersion": this._httpVersion,
			"headers": this._headers,
			"body": this.getBody().toString()
		};
	};

	this.send = function(){
		var body = '';
		if(this._httpServerResponse){
            if(this._statusCode!=304 && this._statusCode!=415){
                
                if(this.getBody()!==''){
                    body = this.getBody().toBuffer();
                    this.setHeader('content-length', this.getBody().length(), true);
                }
				
            }
		// FIXME: trzeba lepiej to obsluzyc
         this.removeHeader('connection');

			this._httpServerResponse.writeHead(this._statusCode, this.getHeaders());
            if(this.getBody()===''){
                this._httpServerResponse.end();
            } else {
                this._httpServerResponse.end(body);
            }
            
            console.log('  -- response send: ' + this._statusCode);
		}
	};

	this._isBinary = function(){
		if(this.isContentType('text/*')|| this.isContentType('application/javascript') || this.isContentType('application/x-javascript') || this.isContentType('application/xhtml+xml') || this.isContentType('application/json')){
			return false;
		}
		return true;
	};

	this._isText = function(contentType){
		return !this._isBinary(contentType);
	};

	this._parseStatusLine = function(statusLine){
		var matches = this._reStatusLine.exec(statusLine);

		return {statusCode: parseInt(matches[2], 10), httpVersion: matches[1]};
	};

	this._statusLineAsString = function(){
		var statusCode = this.getStatusCode();
		return 'HTTP/' + this.getHttpVersion() + ' ' + statusCode + ' ' + http.STATUS_CODES[statusCode];
	};
    
    this.isCachingDisabled = function(){
        var cachingDisabled = false;
        cachingDisabled = cachingDisabled || this.getHeader('pragma') == 'no-cache';
        cachingDisabled = cachingDisabled || this.getHeader('cache-control') == 'no-cache';
        return cachingDisabled;
    };
    
    this.isEmptyBody = function(){
        return this._body === '';
    }
    
    this.isModified = function(request){
        var modifiedSince = request.getHeader('if-modified-since');
        var lastModified = this.getHeader('last-modified');
        
        // check If-Modified-Since
        if (modifiedSince && lastModified) {
            modifiedSince = new Date(modifiedSince);
            lastModified = new Date(lastModified);
            // Ignore invalid dates
            if (!isNaN(modifiedSince.getTime())) {
              if (lastModified <= modifiedSince) return false;
            }
        }
        
        return true;
    };
    
}

Response = new Class(new Response());

Response.Exception = {};
Response.Exception.WRONG_BODY = "Body has to be a string";
Response.Exception.WRONG_STATUS_CODE = "Status code has to be a number";
Response.Exception.WRONG_CONTENT_TYPE = "Content-type must be a string";
Response.Exception.WRONG_CHARSET_TYPE = "Charset must be a string";

exports.Response = Response;

==> core-0.4.1.tgz: package/lib/net/Response.js <==
var Message = require('./Message.js').Message;
var Class = require('../class/Class.js').Class;

var Assertions = require('../common/Assertions.js');
var Types = require('../common/Types.js');
var BinaryData = require('../data/BinaryData.js').BinaryData;

var Event = require('../event/Event.js').Event;

var http = require('http');

var Response = function(){
	this.Extends = Message;

	this._statusCode = null;

	this._cookieKey = 'set-cookie';

	this._httpServerResponse = null;

	this._reCharset = /charset=([A-Za-z\-0-9]+)/i;

	this._reStatusLine = /([0-9]{1,}\.[0-9]{1,}).*([0-9]{3,})/;

	this.initialize = function(params){
		if(Types.isInstanceOf(params, http.ServerResponse)){
			this._httpServerResponse = params;
		}else{
			this.parent(params);
		}
	};

	this.setStatusCode = function(code){
		Assertions.isNumber(code, Response.Exception.WRONG_STATUS_CODE);

		this._statusCode = code;

		return this;
	};

	this.getStatusCode = function(){
		return this._statusCode;
	};

	this._setBody = function(body){
        
		if(Types.isInstanceOf(body, BinaryData)){
			this._body = body;
		}else{
			var encoding, characterEncoding = null;
			if(Types.isInstanceOf(body, Buffer)){
				if(!this._isBinary()){
					encoding = BinaryData.Encoding.TEXT;

					switch(this.getCharset()){
						case 'iso-8859-2':
							characterEncoding = BinaryData.CharacterEncoding.ISO_8859_2;
                            encoding = BinaryData.Encoding.BINARY;
						break;
						case 'utf-8':
						case 'utf8':
							characterEncoding = BinaryData.CharacterEncoding.UTF8
						break;
                        default:
                            // TODO poprawic to bo to nie jest binarka
                            encoding = BinaryData.Encoding.BINARY;
                        break;   
					}
				}else{
					encoding = BinaryData.Encoding.BINARY;
				}
			}else{
				encoding = BinaryData.Encoding.TEXT;
				characterEncoding = BinaryData.CharacterEncoding.UTF8;
			}
			this._body = new BinaryData(body, encoding, characterEncoding);
		}
	};
    
	this.setBody = function(body){
        Assertions.isInstanceOf(body, BinaryData);
		this._body = body;
        return this;            
    };

    /**
     * Indicates if response content type matches given contentType parameter
     * @public
     * @param {String} contentType mime type to match (e.g. text/html, application/*)
     * @returns {Boolean} 
     */
    this.isContentType = function(contentType){
		Assertions.isString(contentType, Response.Exception.WRONG_CONTENT_TYPE);

		var contentHeader = this.getHeader('content-type');
		
		if(contentHeader){
	       var currentContentTypeParts = this.getHeader('content-type').split(";")[0].split("/");
    	   var checkedContentTypeParts = contentType.split(";")[0].split("/");
	       if( 
    	       (checkedContentTypeParts[0] == "*" || checkedContentTypeParts[0] == currentContentTypeParts[0])
        	   &&
	           (checkedContentTypeParts[1] == "*" || checkedContentTypeParts[1] == currentContentTypeParts[1])
    	   ){
        	   return true;            
	       }
		}

		return false;
    };

	this.isCharset = function(charset){
		Assertions.isString(charset, Response.Exception.WRONG_CHARSET_TYPE);

		var currentCharset = this.getCharset();

		if(!Types.isNull(currentCharset) && currentCharset.toLowerCase() == charset.toLowerCase()){
			return true;
		}

		return false;
	};

	this.getCharset = function(){
		var matches = this._reCharset.exec(this.getHeader('content-type'));

		if(matches && !Types.isUndefined(matches[1])){
			return matches[1].toLowerCase();
		}

		return null;
	};

	this.getEncoding = function(){
		if(this._isBinary()){
			return 'base64';
		}

		return this.getCharset();
	};

	this.toJson = function(){
        this.setHeader('content-length', this.getBody().length(), true);
        
		return {
			"statusCode": this._statusCode,
			"httpVersion": this._httpVersion,
			"headers": this._headers,
			"body": this.getBody().toString()
		};
	};

	this.send = function(){
		var body = '';
		if(this._httpServerResponse){
            if(this._statusCode!=304 && this._statusCode!=415){
                if(this.getBody() != ''){
                    body = this.getBody().toBuffer();
                    this.setHeader('content-length', this.getBody().length(), true);
                }
            }

			this.dispatchEvent(new Event(Response.Event.SEND));

			this._httpServerResponse.writeHead(this._statusCode, this.getHeaders());
            if(this.getBody() == ''){
                this._httpServerResponse.end();
            } else {
                this._httpServerResponse.end(body);
            }
            
            console.log('  -- response send: ' + this._statusCode);
		}
	};

	this._isBinary = function(){
		if(this.isContentType('text/*')|| this.isContentType('application/javascript') || this.isContentType('application/x-javascript') || this.isContentType('application/xhtml+xml') || this.isContentType('application/json')){
			return false;
		}
		return true;
	};

	this._isText = function(contentType){
		return !this._isBinary(contentType);
	};

	this._parseStatusLine = function(statusLine){
		var matches = this._reStatusLine.exec(statusLine);

		return {statusCode: parseInt(matches[2], 10), httpVersion: matches[1]};
	};

	this._statusLineAsString = function(){
		var statusCode = this.getStatusCode();
		return 'HTTP/' + this.getHttpVersion() + ' ' + statusCode + ' ' + http.STATUS_CODES[statusCode];
	};

    this.isCachingDisabled = function(){
        var pragma = this.getHeader('pragma');
        var cacheControl = this.getHeader('cache-control');
        
        var cachingDisabled = false;
        cachingDisabled = cachingDisabled || (pragma && pragma.search('no-cache')>-1);
        cachingDisabled = cachingDisabled || (cacheControl && cacheControl.search('no-cache')>-1);
        cachingDisabled = cachingDisabled || (cacheControl && cacheControl.search('private')>-1);
        return cachingDisabled;
    };
    
    
    this.getCacheTimeInSeconds = function(){
        
        
        // jezeli jest pragma: no-cache to nigdy nie cachuj!
        var pragma = this.getHeader('pragma');
        if(pragma && pragma.search('no-cache')>-1){
            return 0;
        }
        
        
        var cacheControl = this.getHeader('cache-control');
        if(cacheControl){
            // jezeli cache-control: no-cache to nigdy nie cachuj!
            if(cacheControl.search('no-cache')>-1){
                return 0;
            }
            // jezeli cache-control: private to nigdy nie cachuj bo mamy shared cache
            if(cacheControl.search('private')>-1){
                return 0;
            }
            // poszukaj czy cache-control ma max-age wtedy zacachuj na max-age
            var maxAge = cacheControl.match(/max-age=([0-9]+)/i);
            if(maxAge && maxAge[1]){
                return maxAge[1]; // ilosc sekund na ile cachujemy
            }
        }
        

        // jezeli jest expires to zacachuj na expires
        var expires = this.getHeader('expires');
        if(expires){
            expires = new Date(expires);
            if(!isNaN(expires.getTime())) {
                var now = new Date();
                var diff = Math.round((expires-now)/1000);
                if(diff>0){
                    return diff;
                }
            }
        }
        
        // jezeli nie ma zadnych naglowkow to nigdy nie cachuj
        return 0;
    };
    
    this.isEmptyBody = function(){
        return this._body === '';
    };
    
    
    this.isExpired = function(){
        var expires = this.getHeader('expires');
        if(expires){
            expires = new Date(expires);
            if (!isNaN(expires.getTime())) {
              var now = new Date();
              if (expires > now) return false;
            }
        }
        return true;
    };
    
    this.isModified = function(request){
        var modifiedSince = request.getHeader('if-modified-since');
        var lastModified = this.getHeader('last-modified');
        
        // check If-Modified-Since
        if (modifiedSince && lastModified) {
            modifiedSince = new Date(modifiedSince);
            lastModified = new Date(lastModified);
            // Ignore invalid dates
            if (!isNaN(modifiedSince.getTime())) {
              if (lastModified <= modifiedSince) return false;
            }
        }
        
        return true;
    };
    
}

Response = new Class(new Response());

Response.Event = {};
Response.Event.SEND = "Response_Event_SEND";

Response.Exception = {};
Response.Exception.WRONG_BODY = "Body has to be a string";
Response.Exception.WRONG_STATUS_CODE = "Status code has to be a number";
Response.Exception.WRONG_CONTENT_TYPE = "Content-type must be a string";
Response.Exception.WRONG_CHARSET_TYPE = "Charset must be a string";

exports.Response = Response;
